REM This file is part of the zBoy project
REM Copyright (C) Mateusz Viste 2010
REM
REM Divider register (DIV) emulation
REM
REM This register is very similar to the Timer register (TIMA), only much easier. The divider register (DIV) is
REM located at memory address 0xFF04 and one of its uses is to supply a seed for a random number generator in
REM games. You can test this by not emulating the divider register and then playing tetris. You will notice that
REM the only object you are given in tetris is the square block which makes the game a bit easy. The register
REM increments itself through values of 0-255 and starts back at 0 again when going one higher than 255. You
REM need to intercept the write to the DIV address. When the game attempts to write to DIV then DIV is set to 0.
REM
REM The DIV register is incrementing 16384 times a second (that is, every 256 clock cycles)
REM

DIM SHARED AS UBYTE DividerCycleCounter = 255

'SUB incDivider(BYREF cpuCyclesDone AS UBYTE)
#MACRO incDivider(cpuCyclesDone)
  IF (DividerCycleCounter < cpuCyclesDone) THEN
      IF (IoRegisters(&hFF04) = 255) THEN
          IoRegisters(&hFF04) = 0
        ELSE
          IoRegisters(&hFF04) += 1
      END IF
      DividerCycleCounter += (256 - cpuCyclesDone)
    ELSE
      DividerCycleCounter -= cpuCyclesDone
  END IF
#ENDMACRO
'END SUB
