REM
REM  Z80 Timer emulation
REM  This file is part of the zBoy project.
REM  Copyright (C) Mateusz Viste 2010
REM

REM FF05 - TIMA - Timer counter (R/W)
REM        This timer is incremented by a clock frequency
REM        specified by the TAC register ($FF07). When the
REM        value overflows (gets bigger than FFh) then it
REM        will be reset to the value specified in TMA
REM        (FF06), and an interrupt will be requested.

DIM SHARED AS UINTEGER TimerC = 0     ' Timer counter


#MACRO CheckTIMA()
  IF (IoRegisters(&hFF05) = 255) THEN
      'MemoryWrite(&hFF05, (IoRegisters(&hFF06)))
      IoRegisters(&hFF05) = IoRegisters(&hFF06)
      'MemoryWrite(&hFF0F, IoRegisters(&hFF0F) OR &b00000100)  ' Request an interrupt via bit 2 of the IF
      IoRegisters(&hFF0F) = IoRegisters(&hFF0F) OR &b00000100
    ELSE
      'MemoryWrite(&hFF05, (IoRegisters(&hFF05) + 1))
      IoRegisters(&hFF05) += 1
  END IF
#ENDMACRO


'SUB uTimer(BYREF cycles AS UBYTE)           ' Timer Emulation
#MACRO uTimer(cycles)
  IF (IoRegisters(&hFF07) AND &b00000100) > 0 THEN   ' test TAC bit 2 (start/stop timer)
    TimerC += cycles
    SELECT CASE (IoRegisters(&hFF07) AND &b00000011) ' bits 1+0 -> input clock select
      CASE &b00  ' 4.096 KHz
        IF (TimerC > 1023) THEN
          TimerC -= 1024
          CheckTIMA()
        END IF
      CASE &b01  ' 262.144 KHz
        IF (TimerC > 15) THEN
          TimerC -= 16
          CheckTIMA()
        END IF
      CASE &b10  ' 65.536 KHz
        IF (TimerC > 63) THEN
          TimerC -= 64
          CheckTIMA()
        END IF
      CASE &b11  ' 16.384 KHz
        IF (TimerC > 255) THEN
          TimerC -= 256
          CheckTIMA()
        END IF
    END SELECT
  END IF
'END SUB
#ENDMACRO
