REM
REM  This file is part of the zBoy project
REM  Copyright (C) Mateusz Viste 2010
REM

#INCLUDE ONCE "crc32.bi"

FUNCTION CheckChecksums(BYREF RomSize AS UINTEGER) AS UBYTE
  DIM AS INTEGER x, CmpChecksum, TotalChecksum
  DIM AS UBYTE tByteBuff, Result = 0
  REM Returns 0 if GameBoy checksums are okay, non-zero otherwise.
  REM 14Dh: A complement check is computed as the sum of the bytes in the header of the program subtracted by one and then two's complemented. The complement check is required by the NINTENDO GAMEBOY device.
  FOR x = &h134 TO &h14C
    tByteBuff = MemoryROM(x)
    CmpChecksum = (CmpChecksum + tByteBuff) AND &b11111111
  NEXT x
  CmpChecksum = (231 - CmpChecksum)
  IF (CmpChecksum < 0) THEN  ' If negative, get the 2-complement
    CmpChecksum = ABS(CmpChecksum)
    CmpChecksum = (CmpChecksum XOR &b11111111)
    CmpChecksum += 1
  END IF
  IF (MemoryROM(&h14D) = CmpChecksum) THEN
      REM 014E-014F: Checksum (higher byte first) produced by adding all bytes of a cartridge except for two checksum bytes and taking two lower bytes of the result. (GameBoy ignores this value.)
      FOR x = 0 TO RomSize - 1
        tByteBuff = MemoryROM(x)
        IF (x <> &h14E) AND (x <> &h14F) THEN TotalChecksum += tByteBuff
        TotalChecksum = (TotalChecksum AND &b1111111111111111) ' Keep the checksum 32bit wide
      NEXT x
      tByteBuff = MemoryROM(&h14E)
      IF tByteBuff <> ((TotalChecksum SHR 8) AND &b11111111) THEN Result = 3 ' Check first byte of checksum
      tByteBuff = MemoryROM(&h14F)
      IF tByteBuff <> (TotalChecksum AND &b11111111) THEN Result = 2         ' Check second byte of checksum
    ELSE
      Result = 1
  END IF
  RETURN Result
END FUNCTION


SUB LoadRom(BYREF RomFilepath AS STRING)
  DIM NintendoSignature(&h104 TO &h133) AS UBYTE => {&hCE,&hED,&h66,&h66,&hCC,&h0D,&h00,&h0B,_
                                                     &h03,&h73,&h00,&h83,&h00,&h0C,&h00,&h0D,_
                                                     &h00,&h08,&h11,&h1F,&h88,&h89,&h00,&h0E,_
                                                     &hDC,&hCC,&h6E,&hE6,&hDD,&hDD,&hD9,&h99,_
                                                     &hBB,&hBB,&h67,&h63,&h6E,&h0E,&hEC,&hCC,_
                                                     &hDD,&hDC,&h99,&h9F,&hBB,&hB9,&h33,&h3E}
  DIM AS INTEGER RomHandler, x, OpenResult
  DIM AS UINTEGER RomSize
  DIM AS STRING ShortRomFilename, RomDataForCRC, RomFile
  RomFile = RomFilepath
  ShortRomFilename = RomFile
  IF INSTR(ShortRomFilename, "/") > 0 THEN
    ShortRomFilename = MID(ShortRomFilename, INSTRREV(ShortRomFilename, "/") + 1)
  END IF
  IF INSTR(ShortRomFilename, "\") > 0 THEN
    ShortRomFilename = MID(ShortRomFilename, INSTRREV(ShortRomFilename, "\") + 1)
  END IF
  RomInfos.Filename = ShortRomFilename
  RomHandler = FREEFILE
  IF FileExists(RomFile) = 0 THEN
      PrintMsg("ERROR: THE SPECIFIED ROM FILE DOESN'T EXISTS!")
      PressAnyKey() : END
    ELSE
      OpenResult = OPEN(RomFile, FOR BINARY, AS #RomHandler)
      IF (OpenResult = 0) THEN
        RomSize = LOF(RomHandler)
        IF (RomSize < 16384) THEN
            CLOSE #RomHandler
            PrintMsg("ERROR: THE FILE IS TOO SMALL TO BE A VALID GAMEBOY ROM!")
            PressAnyKey() : END
          ELSE
            IF (RomSize > 4194304) THEN
                CLOSE #RomHandler
                PrintMsg("ERROR: THE FILE IS TOO BIG TO BE A VALID GAMEBOY ROM!")
                PressAnyKey() : END
              ELSE
                #IFNDEF EMBEDROM
                  REDIM MemoryROM(0 TO RomSize - 1) AS UBYTE   ' redim the ROM memory table
                #ENDIF
                FOR x = 0 TO (RomSize - 1)
                  GET #RomHandler, x + 1, MemoryROM(x)
                  RomDataForCRC += CHR(MemoryROM(x))
                NEXT x
                CLOSE #RomHandler
            END IF
          END IF
        ELSE
          PrintMsg("ERROR: THE SPECIFIED ROM FILE COULD NOT BE OPENED!")
          PressAnyKey() : END
      END IF
  END IF

  RomInfos.CrcSum = CRC32(RomDataForCRC, RomSize)    ' Calculate CRC32 value of the ROM

  REM  Check for ROM signature
  FOR x = &h104 TO &h133
    IF (MemoryROM(x) <> NintendoSignature(x)) THEN
      PrintMsg("ERROR: THE FILE IS NOT A VALID GAMEBOY ROM!")
      PressAnyKey() : END
    END IF
  NEXT x
  SELECT CASE CheckChecksums(RomSize)
    CASE 0
      REM Nothing here, everything is ok.
    CASE 1, 2, 3
      UserMessage = "WARNING: CHECKSUM ERROR!" + CHR(10) + "THIS ROM IS PROBABLY CORRUPTED."
    CASE ELSE
      PrintMsg("ERROR: AN UNEXPECTED CONDITION OCCURED WHEN CHECKING ROM'S CHECKSUMS!")
      PressAnyKey() : END
  END SELECT
END SUB
